@@ -122,7 +122,7 @@
   });
 
   QUnit.test('update index when id changes', function(assert) {
-    assert.expect(4);
+    assert.expect(6);
     var collection = new Backbone.Collection();
     collection.add([
       {id: 0, name: 'one'},
@@ -130,7 +130,11 @@
     ]);
     var one = collection.get(0);
     assert.equal(one.get('name'), 'one');
-    collection.on('change:name', function(model) { assert.ok(this.get(model)); });
+    collection.on('change:name', function(model) {
+      assert.ok(this.get(model));
+      assert.equal(model, this.get(101));
+      assert.equal(this.get(0), null);
+    });
     one.set({name: 'dalmatians', id: 101});
     assert.equal(collection.get(0), null);
     assert.equal(collection.get(101).get('name'), 'dalmatians');
@@ -750,11 +754,13 @@
     var wrapped = col.chain();
     assert.equal(wrapped.map('id').max().value(), 3);
     assert.equal(wrapped.map('id').min().value(), 0);
-    assert.deepEqual(wrapped
+    assert.deepEqual(
+      wrapped
       .filter(function(o){ return o.id % 2 === 0; })
       .map(function(o){ return o.id * 2; })
       .value(),
-      [4, 0]);
+      [4, 0]
+    );
     assert.deepEqual(col.difference([c, d]), [a, b]);
     assert.ok(col.includes(col.sample()));
 
@@ -795,7 +801,7 @@
     assert.deepEqual(coll.sortBy('a')[3], model);
     assert.deepEqual(coll.sortBy('e')[0], model);
     assert.deepEqual(coll.countBy({a: 4}), {'false': 3, 'true': 1});
-    assert.deepEqual(coll.countBy('d'), {'undefined': 4});
+    assert.deepEqual(coll.countBy('d'), {undefined: 4});
     assert.equal(coll.findIndex({b: 1}), 0);
     assert.equal(coll.findIndex({b: 9}), -1);
     assert.equal(coll.findLastIndex({b: 1}), 3);
@@ -1478,7 +1484,7 @@
     assert.expect(3);
     var collection = new (Backbone.Collection.extend({
       comparator: function(m1, m2) {
-        return m1.get('a') > m2.get('a') ? 1 : (m1.get('a') < m2.get('a') ? -1 : 0);
+        return m1.get('a') > m2.get('a') ? 1 : m1.get('a') < m2.get('a') ? -1 : 0;
       }
     }))([{id: 1}, {id: 2}, {id: 3}]);
     collection.on('sort', function() { assert.ok(true); });
@@ -1688,12 +1694,20 @@
 
   QUnit.test('modelId', function(assert) {
     var Stooge = Backbone.Model.extend();
-    var StoogeCollection = Backbone.Collection.extend({model: Stooge});
+    var StoogeCollection = Backbone.Collection.extend();
 
-    // Default to using `Collection::model::idAttribute`.
+    // Default to using `id` if `model::idAttribute` and `Collection::model::idAttribute` not present.
     assert.equal(StoogeCollection.prototype.modelId({id: 1}), 1);
+
+    // Default to using `model::idAttribute` if present.
     Stooge.prototype.idAttribute = '_id';
+    var model = new Stooge({_id: 1});
+    assert.equal(StoogeCollection.prototype.modelId(model.attributes, model.idAttribute), 1);
+
+    // Default to using `Collection::model::idAttribute` if model::idAttribute not present.
+    StoogeCollection.prototype.model = Stooge;
     assert.equal(StoogeCollection.prototype.modelId({_id: 1}), 1);
+
   });
 
   QUnit.test('Polymorphic models work with "simple" constructors', function(assert) {
@@ -1748,7 +1762,7 @@
     assert.equal(collection.at(1), collection.get('b-1'));
   });
 
-  QUnit.test('Collection with polymorphic models receives default id from modelId', function(assert) {
+  QUnit.test('Collection with polymorphic models receives id from modelId using model instance idAttribute', function(assert) {
     assert.expect(6);
     // When the polymorphic models use 'id' for the idAttribute, all is fine.
     var C1 = Backbone.Collection.extend({
@@ -1761,7 +1775,8 @@
     assert.equal(c1.modelId({id: 1}), 1);
 
     // If the polymorphic models define their own idAttribute,
-    // the modelId method should be overridden, for the reason below.
+    // the modelId method will use the model's idAttribute property before the
+    // collection's model constructor's.
     var M = Backbone.Model.extend({
       idAttribute: '_id'
     });
@@ -1771,12 +1786,12 @@
       }
     });
     var c2 = new C2({_id: 1});
-    assert.equal(c2.get(1), void 0);
-    assert.equal(c2.modelId(c2.at(0).attributes), void 0);
+    assert.equal(c2.get(1), c2.at(0));
+    assert.equal(c2.modelId(c2.at(0).attributes, c2.at(0).idAttribute), 1);
     var m = new M({_id: 2});
     c2.add(m);
-    assert.equal(c2.get(2), void 0);
-    assert.equal(c2.modelId(m.attributes), void 0);
+    assert.equal(c2.get(2), m);
+    assert.equal(c2.modelId(m.attributes, m.idAttribute), 2);
   });
 
   QUnit.test('Collection implements Iterable, values is default iterator function', function(assert) {
@@ -2100,4 +2115,37 @@
     var collection = new Backbone.Collection([model]);
     assert.ok(collection.get(model));
   });
+
+  QUnit.test('#4233 - can instantiate new model in ES class Collection', function(assert) {
+    var model;
+    try {
+      model = new Function('return ({\n' +
+          '    model(attrs, options) {\n' +
+          '        var MyModel = Backbone.Model.extend({});\n' +
+          '        return new MyModel(attrs, options);\n' +
+          '    }\n' +
+          '}).model')();
+    } catch (error) {
+      model = error;
+    }
+
+    if (model instanceof SyntaxError) {
+      assert.expect(0);
+      return;
+    }
+
+    assert.expect(1);
+
+    var MyCollection = Backbone.Collection.extend({
+      modelId: function(attr) {
+        return attr.x;
+      },
+
+      model: model
+    });
+
+    var instance = new MyCollection([{a: 2}]);
+
+    assert.ok(instance, 'Should instantiate collection with model');
+  });
 })(QUnit);
